from datetime import datetime

from django.conf import settings
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404, render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import simplejson
from django.core.mail import EmailMessage

from staff.models import StaffMember
from staff.forms import ContactForm

def index(request, template_name='staffmembers/index.html'):
    """
    The list of employees or staff members
    """
    return render_to_response(template_name,
                              {'staff': StaffMember.objects.active()},
                              context_instance=RequestContext(request))

def userinfo_json(request, user_id):
    """
    Return the user's information in a json object
    """
    data = {'first_name': '',
            'last_name': '',
            'email': '',
            'slug': '',
            'bio': '',
            'phone': '',
            'is_active': False}
          
    try:
        member = StaffMember.objects.get(pk=user_id)
        for key in data.keys():
            if hasattr(member, key):
                data[key] = getattr(member, key, '')
                
    except StaffMember.DoesNotExist:
        pass
        
    return HttpResponse(simplejson.dumps(data),
                        mimetype='application/json')
                              
def contact(request, slug, template_name='staffmembers/contact.html', 
    success_url='/staff/contact/done/', 
    email_subject_template='staffmembers/emails/subject.txt', 
    email_body_template='staffmembers/emails/body.txt'):
    """
    Handle a contact request
    """
    member = get_object_or_404(StaffMember, slug__iexact=slug, is_active=True)
    
    if request.method == 'POST':
        form = ContactForm(request.POST)
        if form.is_valid():
            subject = render_to_string(
                email_subject_template, 
                {'member': member}
            )
            subject = ''.join(subject.splitlines())
            
            message = render_to_string(
                email_body_template,
                {'name': form.cleaned_data['name'],
                'email': form.cleaned_data['email'],
                'message': form.cleaned_data['message']}
            )
            
            EmailMessage(subject, message, settings.DEFAULT_FROM_EMAIL, [member.email],
                    headers = {'Reply-To': form.cleaned_data['email']}).send()
            
            return HttpResponseRedirect(success_url)
    else:
        initial = {}
        if not request.user.is_anonymous():
            initial = {'name': '%s %s' % (
                            request.user.first_name, request.user.last_name),
                       'email': request.user.email}
        form = ContactForm(initial=initial)
    
    return render_to_response(template_name,
                              {'form': form,
                               'member': member},
                              context_instance=RequestContext(request))

# TODO: This needs to be refactored so that any other model can be referenced
def story_archive(request, slug, 
    template_name='staffmembers/story_archive.html'):
    """
    Return the list of stories written by this staff member
    """
    member = get_object_or_404(StaffMember, slug__iexact=slug, is_active=True)
    
    stories = []
    if hasattr(member, 'story_set'):
        from story.settings import PUBLISHED_STATUS
        stories = member.story_set.filter(publish_date__lte=datetime.now()
            ).filter(status__exact=PUBLISHED_STATUS
            ).order_by('-publish_date')
    
    return render_to_response(template_name,
                              {'member': member,
                               'stories': stories},
                              context_instance=RequestContext(request))
    


# Changing the staff member's first name, last name or email should change the user's as well
# Changing the staff member's first name or last name should change the slug
#   It would really be cool if you changed the slug, you check if redirects is
#   enabled and insert a redirect from the old slug to the new slug
# Changing the user's first name, last name or email should change the staff member's as well
# Changing the user's first name or last name should change the staff member's slug

from django.test import TestCase
from django.contrib.auth.models import User
from staff.models import StaffMember
from django.template.defaultfilters import slugify

class StaffTest(TestCase):
    """Tests for the Staff model"""
    def setUp(self):
        self.user = User.objects.create_user("tempuser", "tempuser", "temp@user.com")
        self.user.first_name = "Temporary"
        self.user.last_name = "User"
        self.user.save()
    
    def assertStaffInfoEqual(self, staffmember, user):
        self.assertEqual(staffmember.first_name, user.first_name)
        self.assertEqual(staffmember.last_name, user.last_name)
        self.assertEqual(staffmember.email, user.email)
        self.assertEqual(staffmember.slug, slugify('%s %s' % (user.first_name, user.last_name)))
    
    def testAutoCreation(self):
        self.assertEqual(StaffMember.objects.all().count(), 0)
        self.user.is_staff = True
        self.user.save()
        self.assertEqual(StaffMember.objects.all().count(), 1)
        s = self.user.staffmember_set.all()[0]
        self.assertStaffInfoEqual(s, self.user)
    
    def testAutoDeactivation(self):
        self.assertEqual(StaffMember.objects.all().count(), 0)
        self.user.is_staff = True
        self.user.save()
        self.assertEqual(StaffMember.objects.all().count(), 1)
        self.user.is_staff = False
        self.user.save()
        self.assertEqual(StaffMember.objects.all().count(), 0)
        self.assertEqual(StaffMember.objects.inactive().count(), 1)
        s = self.user.staffmember_set.inactive()[0]
        self.assertEqual(s.is_active, False)
    
    def testUserInfoChange(self):
        self.user.is_staff = True
        self.user.save()
        s = self.user.staffmember_set.all()[0]
        self.assertStaffInfoEqual(s, self.user)
        self.user.first_name = "Permanent"
        self.user.save()
        s = self.user.staffmember_set.all()[0]
        self.assertStaffInfoEqual(s, self.user)


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding field 'StaffMember.photo_height'
        db.add_column('staff_staffmember', 'photo_height', self.gf('django.db.models.fields.IntegerField')(default=0, blank=True), keep_default=False)

        # Adding field 'StaffMember.photo_width'
        db.add_column('staff_staffmember', 'photo_width', self.gf('django.db.models.fields.IntegerField')(default=0, blank=True), keep_default=False)

        # Adding field 'StaffMember.twitter'
        db.add_column('staff_staffmember', 'twitter', self.gf('django.db.models.fields.CharField')(default='', max_length=100, blank=True), keep_default=False)

        # Adding field 'StaffMember.website'
        db.add_column('staff_staffmember', 'website', self.gf('django.db.models.fields.URLField')(default='', max_length=200, blank=True), keep_default=False)

        # Changing field 'StaffMember.photo'
        db.alter_column('staff_staffmember', 'photo', self.gf('staff.fields.RemovableImageField')(max_length=100))


    def backwards(self, orm):
        
        # Deleting field 'StaffMember.photo_height'
        db.delete_column('staff_staffmember', 'photo_height')

        # Deleting field 'StaffMember.photo_width'
        db.delete_column('staff_staffmember', 'photo_width')

        # Deleting field 'StaffMember.twitter'
        db.delete_column('staff_staffmember', 'twitter')

        # Deleting field 'StaffMember.website'
        db.delete_column('staff_staffmember', 'website')

        # Changing field 'StaffMember.photo'
        db.alter_column('staff_staffmember', 'photo', self.gf('staff.fields.RemovableImageField')(max_length=100))


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'staff.staffmember': {
            'Meta': {'ordering': "('last_name', 'first_name')", 'object_name': 'StaffMember'},
            'bio': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '150', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '150', 'null': 'True', 'blank': 'True'}),
            'phone': ('django.contrib.localflavor.us.models.PhoneNumberField', [], {'max_length': '20', 'blank': 'True'}),
            'photo': ('staff.fields.RemovableImageField', [], {'max_length': '100', 'blank': 'True'}),
            'photo_height': ('django.db.models.fields.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'photo_width': ('django.db.models.fields.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['sites.Site']", 'symmetrical': 'False'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50', 'db_index': 'True'}),
            'twitter': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True'}),
            'website': ('django.db.models.fields.URLField', [], {'max_length': '200', 'blank': 'True'})
        }
    }

    complete_apps = ['staff']



from django import forms
from django.utils.translation import ugettext_lazy as _

from models import StaffMember

class StaffMemberForm(forms.ModelForm):
    """
    Basic form for entering data about the staff member. Changes the bio
    field to a tinyMCE widget if specified.
    """
    # bio = forms.CharField(widget=WIDGET)
    
    class Meta:
        model = StaffMember
        exclude = ('slug', 'first_name', 'last_name')


class ContactForm(forms.Form):
    """
    A simple contact form to allow contacting the staff
    """
    name = forms.CharField(label=_('Your Name'))
    email = forms.EmailField(label=_('Your Email'))
    message = forms.CharField(label=_('Message'), widget=forms.Textarea)


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'StaffMember'
        db.create_table('staff_staffmember', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], unique=True)),
            ('first_name', self.gf('django.db.models.fields.CharField')(max_length=150, null=True, blank=True)),
            ('last_name', self.gf('django.db.models.fields.CharField')(max_length=150, null=True, blank=True)),
            ('slug', self.gf('django.db.models.fields.SlugField')(unique=True, max_length=50, db_index=True)),
            ('email', self.gf('django.db.models.fields.EmailField')(max_length=75, null=True, blank=True)),
            ('bio', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('is_active', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('phone', self.gf('django.contrib.localflavor.us.models.PhoneNumberField')(max_length=20, blank=True)),
            ('photo', self.gf('staff.fields.RemovableImageField')(max_length=100, blank=True)),
        ))
        db.send_create_signal('staff', ['StaffMember'])

        # Adding M2M table for field sites on 'StaffMember'
        db.create_table('staff_staffmember_sites', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('staffmember', models.ForeignKey(orm['staff.staffmember'], null=False)),
            ('site', models.ForeignKey(orm['sites.site'], null=False))
        ))
        db.create_unique('staff_staffmember_sites', ['staffmember_id', 'site_id'])


    def backwards(self, orm):
        
        # Deleting model 'StaffMember'
        db.delete_table('staff_staffmember')

        # Removing M2M table for field sites on 'StaffMember'
        db.delete_table('staff_staffmember_sites')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'staff.staffmember': {
            'Meta': {'ordering': "('last_name', 'first_name')", 'object_name': 'StaffMember'},
            'bio': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'null': 'True', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '150', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '150', 'null': 'True', 'blank': 'True'}),
            'phone': ('django.contrib.localflavor.us.models.PhoneNumberField', [], {'max_length': '20', 'blank': 'True'}),
            'photo': ('staff.fields.RemovableImageField', [], {'max_length': '100', 'blank': 'True'}),
            'sites': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['sites.Site']", 'symmetrical': 'False'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '50', 'db_index': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'unique': 'True'})
        }
    }

    complete_apps = ['staff']



"""
Admin classes for the StaffMember model
"""

from django.contrib import admin
from django.contrib.auth.models import User
from django.contrib.auth.admin import UserAdmin
from django.forms.models import inlineformset_factory

# from staff.forms import StaffMemberForm
from staff.models import StaffMember

class StaffMemberAdmin(admin.StackedInline):
    """
    Admin form for a StaffMember that won't appear if the associated User 
    isn't actually a staff member.
    """
    # form = StaffMemberForm
    fieldsets = (
        ('Personal Info', {'fields': ('bio', 'photo', 'twitter', 'website', 
            'phone',)}),
        ('Responsibilities', {'fields': ('sites',)}),
    )
    filter_horizontal = ('sites',)
    model = StaffMember
    max_num = 1
    
    def get_formset(self, request, obj=None, **kwargs):
        """
        Return a form, if the obj has a staffmember object, otherwise 
        return an empty form
        """
        if obj is not None and obj.staffmember_set.all().count():
            return super(StaffMemberAdmin, self).get_formset(
                request, 
                obj, 
                **kwargs
            )
        
        defaults = {
            "exclude": None,
            "extra": 0,
            "max_num": 0,
        }
        return inlineformset_factory(self.parent_model, self.model, **defaults)

class StaffUserAdmin(UserAdmin):
    """
    Subclasses the UserAdmin to add the staffmember as an inline.
    """
    inlines = [StaffMemberAdmin,]

admin.site.unregister(User)
admin.site.register(User, StaffUserAdmin)


from django import forms
from django.conf import settings
from django.db import models
from django.db.models.fields.files import ImageFieldFile
from django.utils.translation import ugettext as _
from django.core.files.images import get_image_dimensions

import os

URL_PREFIX = getattr(settings, 'STATIC_URL', settings.MEDIA_URL)

class DeleteCheckboxWidget(forms.CheckboxInput):
    """
    A specialzed checkbox that displays the filepath or preview and a
    checkbox to indicate that the file should be removed.
    """
    def __init__(self, *args, **kwargs):
        self.is_image = kwargs.pop('is_image')
        self.value = kwargs.pop('initial')
        super(DeleteCheckboxWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        value = value or self.value
        if value:
            html = u'<label for="%s">%s %s</label>' % (
                    attrs['id'],
                    super(DeleteCheckboxWidget, self).render(
                        name, False, attrs
                    ),
                    _('Delete')
                )
            if self.is_image:
                html += u'<br><img src="%s%s" width="50">' % (
                    URL_PREFIX, value
                )
            else:
                html += u'<br><a href="%s%s">%s</a>' % (
                    URL_PREFIX, value, os.path.basename(value)
                )
            return html
        else:
            return u''
    
    def _has_changed(self, initial, data):
        return data


class RemovableFileFormWidget(forms.MultiWidget):
    """
    A widget that shows the file input/upload widget and a delete checkbox
    for removing the file from the file storage
    """
    def __init__(self, is_image=False, initial=None, **kwargs):
        widgets = (forms.FileInput(), DeleteCheckboxWidget(
            is_image=is_image, 
            initial=initial)
        )
        super(RemovableFileFormWidget, self).__init__(widgets)

    def decompress(self, value):
        return [None, value]


class RemovableFileFormField(forms.MultiValueField):
    """
    A field for forms that allows you to remove the file from the storage 
    system within the admin
    """
    widget = RemovableFileFormWidget
    field = forms.FileField
    is_image = False

    def __init__(self, *args, **kwargs):
        fields = [
            self.field(*args, **kwargs), 
            forms.BooleanField(required=False)
        ]
        # Compatibility with form_for_instance
        if kwargs.get('initial'):
            initial = kwargs['initial']
        else:
            initial = None
        self.widget = self.widget(is_image=self.is_image, initial=initial)
        super(RemovableFileFormField, self).__init__(
            fields, label=kwargs.pop('label'), required=False
        )

    def compress(self, data_list):
        return data_list


class RemovableImageFormField(RemovableFileFormField):
    """
    A specific form of image field that allows removal of the original file
    """
    field = forms.ImageField
    is_image = True


class RemovableFileField(models.FileField):
    """
    A field that allows you to remove the file from the storage system within
    the admin.
    """
    def delete_file(self, instance, sender, **kwargs):
        if getattr(instance, self.attname):
            image = getattr(instance, '%s' % self.name)
            file_name = image.path
            # If the file exists and no other object of this type references 
            # it, delete it from the filesystem.
            instance_mgr = instance.__class__._default_manager
            filter_key = '%s__exact' % self.name
            filter_val = getattr(instance, self.attname)
            instance_pk = instance._get_pk_val()
            if os.path.exists(file_name) and not instance_mgr.filter(
                **{filter_key: filter_val}).exclude(pk=instance_pk):
                os.remove(file_name)
    
    def get_internal_type(self):
        return 'FileField'
    
    def save_form_data(self, instance, data):
        if data and data[0]: # Replace file
            self.delete_file(instance, self)
            super(RemovableFileField, self).save_form_data(instance, data[0])
        if data and data[1]: # Delete file
            self.delete_file(instance, self)
            setattr(instance, self.name, None)
    
    def formfield(self, **kwargs):
        defaults = {'form_class': RemovableFileFormField}
        defaults.update(kwargs)
        return super(RemovableFileField, self).formfield(**defaults)
    
    def south_field_triple(self):
        "Returns a suitable description of this field for South."
        # We'll just introspect the _actual_ field.
        from south.modelsinspector import introspector
        field_class = "staff.fields.RemoveableFileField"
        args, kwargs = introspector(self)
        return (field_class, args, kwargs)

class MyImageFieldFile(ImageFieldFile):
    def _get_image_dimensions(self):
        if not hasattr(self, '_dimensions_cache'):
            try:
                close = self.closed
                self.open()
                self._dimensions_cache = get_image_dimensions(self, close=close)
            except IOError:
                self._dimensions_cache = (100, 100)
        return self._dimensions_cache

class RemovableImageField(models.ImageField, RemovableFileField):
    """
    A specific form of removeable file field, but specifically for images
    """
    attr_class = MyImageFieldFile
    
    def formfield(self, **kwargs):
        defaults = {'form_class': RemovableImageFormField}
        defaults.update(kwargs)
        return super(RemovableImageField, self).formfield(**defaults)
